local addon, private = ...
local core = LibStub("AceAddon-3.0"):NewAddon("addon", "AceTimer-3.0", "AceEvent-3.0")
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("addon")
local strformat, pairs, strupper, select, type = string.format, pairs, strupper, select, type
local UnitName, UnitClass, GetItemCount, GetItemIcon = UnitName, UnitClass, GetItemCount, GetItemIcon
local oldTime
local charsList = {}
local playerName, playerClass = UnitName("player"), strupper(select(2,UnitClass("player")))
local MAX_GUILDBANK_SLOTS_PER_TAB = 98

local defaults = {
    profile = {
        modules = {['*'] = false},
        tracking = {['*'] = false},
        showShort = false,
        showIcons = false,
        iconsOnly = false,
        tooltipIcons = false,
        tooltipIconsOnly = false,
        tooltipShowShort = false,
        showZero = true,
        showBank = true,
        showAlts = true,
        showGuild = true,		
        updatetime = 5,
        scale = 1,
        scroll = 900,
    },
    realm = {
        chars = {
            ['*'] = {
                data = {
                    ['*'] = {inv = 0, bank = 0, guild = 0},
                },
                tracking = {
                    ['*'] = false
                },
            }
        }
    }
}

function core:OnInitialize()
    self.charName = playerName
    self.db = LibStub("AceDB-3.0"):New(addon.."_DB", defaults, true)
    db = self.db.profile
    chardb = self.db.realm.chars[self.charName]
    charsdb = self.db.realm.chars
    if not chardb.class then
        chardb.class = playerClass
    end
    for name in pairs(charsdb) do
            charsList[name] = name
    end
    oldTime = db.updatetime
    self:SetupOptions()
end

private.charsList = charsList

function core:OnEnable()
    self.db.RegisterCallback(self, "OnProfileChanged", "applySettings")
    self.db.RegisterCallback(self, "OnProfileCopied", "applySettings")
    self.db.RegisterCallback(self, "OnProfileReset", "applySettings")
    if not self.updateTimer then
        self:setTimer()
    end
    self:applySettings()
end

function core:applySettings()
    if db.updatetime ~= oldTime then
        self:cancelTimer()
        self:setTimer()
        oldTime = db.updatetime
    end
    core:onTextUpdate()
end

function core:setTimer()
    self.updateTimer = self:ScheduleRepeatingTimer("countInv", db.updatetime)
end

function core:cancelTimer()
    self:CancelTimer(self.updateTimer)
end

function core:combatRestart()
    self:onTextUpdate()
    self:setTimer()
end

function core:combatCancel()
    private.bcm.text = L["Paused for Combat!"]
    self:cancelTimer()
end

function core:countGuild()
    for k,itemid in pairs (private.bcitems.id) do
        self:guildBank(itemid)
    end
    for k,itemid in pairs (private.lkitems.id) do
        self:guildBank(itemid)
    end
    for k,itemid in pairs (private.cataitems.id) do
        self:guildBank(itemid)
    end
    for k,itemid in pairs (private.mopitems.id) do
        self:guildBank(itemid)
    end
    self:onTextUpdate()
end

core:RegisterEvent("PLAYER_REGEN_ENABLED", "combatRestart")
core:RegisterEvent("PLAYER_REGEN_DISABLED", "combatCancel")
core:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED", "countGuild")

function core:countInv()
    for k,itemid in pairs (private.bcitems.id) do
        self:countSave(itemid)
    end
    for k,itemid in pairs (private.lkitems.id) do
        self:countSave(itemid)
    end
    for k,itemid in pairs (private.cataitems.id) do
        self:countSave(itemid)
    end
    for k,itemid in pairs (private.mopitems.id) do
        self:countSave(itemid)
    end
    self:onTextUpdate()
end

function core:guildBank(itemid)
local total = 0
    local numTabs = GetNumGuildBankTabs()
    for tab = 1, numTabs do
        local name, icon, isViewable, canDeposit, numWithdrawals, remainingWithdrawals = GetGuildBankTabInfo(tab)
        if isViewable then
            for slot = 1, MAX_GUILDBANK_SLOTS_PER_TAB do
                local texture, count, locked = GetGuildBankItemInfo(tab, slot)
                if count > 0 and locked == nil then
                    local link = GetGuildBankItemLink(tab, slot)
                    if link ~= nil then
                    local id = link:match('|Hitem:(%d+):[%d:]+|h%[(.-)%]|h')
                    id = tonumber(id)
                        if id == itemid then
                            total = total + count
                        end
                    end
                end
            end
        end
    end
    chardb.data[itemid].guild = total or 0
end

function core:countSave(itemid)
    local itemid = itemid
    local inv = GetItemCount(itemid)
    local bank = GetItemCount(itemid, true)
    bank = bank - inv
    chardb.data[itemid].inv = inv or 0
    chardb.data[itemid].bank = bank or 0
end

function core:CharHasElement(itemid)
    if chardb.data[itemid].inv > 0 or (db.showBank and chardb.data[itemid].bank > 0) or (db.showGuild and chardb.data[itemid].guild > 0)then
        return true
    end
    if db.showAlts then
        for char, _ in pairs(charsdb) do
            if charsdb[char].data[itemid].inv > 0 or (db.showBank and charsdb[char].data[itemid].bank > 0) or (db.showGuild and charsdb[char].data[itemid].guild > 0) then
                return true
            end
        end
    end
    return false
end

function core:CountString(itemid, char)
    return self:InvCountString(itemid, char)..self:BankCountString(itemid, char)..self:GuildCountString(itemid, char)
end

function core:InvCountString(itemid, char)
    local tempChar = chardb
    if char then
        tempChar = charsdb[char]
    end
    return self:Color(tempChar.data[itemid].inv)
end

function core:BankCountString(itemid, char)
    local tempChar = chardb
    if char then
        tempChar = charsdb[char]
    end
    if db.showBank then
        return " ( "..self:Color(tempChar.data[itemid].bank).." )"
    else
        return " "
    end
end

function core:GuildCountString(itemid, char)
    local tempChar = chardb
    if char then
        tempChar = charsdb[char]
    end
    if db.showGuild then
        return " { "..self:Color(tempChar.data[itemid].guild).." }"
    else
        return " "
    end
end

function core:Color(count)
    local perc = 0
    local max = 10
    if max ~= 0 then
        perc = count / max
    end
    local r1, g1, b1
    local r2, g2, b2
    if perc <= 0.5 then
        perc = perc * 2
        r1, g1, b1 = 1, 0, 0
        r2, g2, b2 = 1, 1, 0
    else
        perc = perc * 2 - 1
        r1, g1, b1 = 1, 1, 0
        r2, g2, b2 = 0, 1, 0
    end
    local r, g, b = r1 + (r2 - r1)*perc, g1 + (g2 - g1)*perc, b1 + (b2 - b1)*perc
    if r < 0 then
        r = 0
    elseif r > 1 then
        r = 1
    end
    if g < 0 then
        g = 0
    elseif g > 1 then
        g = 1
    end
    if b < 0 then
        b = 0
    elseif b > 1 then
        b = 1
    end
    r, g, b =  r * 255, g * 255, b * 255
    color = strformat("%02x%02x%02x", r, g, b)
    count = ("|cff"..color..count.."|r")
    return count
end